unit fMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, OleCtrls, SHDocVw, StdCtrls;

type
  TForm1 = class(TForm)
    WebBrowser1: TWebBrowser;
    btnLoad: TButton;
    btnInjectJQuery: TButton;
    Memo1: TMemo;
    Button1: TButton;
    Button2: TButton;
    lblJS: TLabel;
    cbJS: TComboBox;
    procedure btnLoadClick(Sender: TObject);
    procedure btnInjectJQueryClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure cbJSKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure SetDropDownWidthToItems(Sender: TObject);
  private
    { Private declarations }
    Yet: Boolean;

    function RetrieveJQueryFromResource: String;

    procedure ExecJS(const aCode: String; const output: Boolean = true);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses
   MSHTML, ActiveX;

   procedure WBLoadHTML(WebBrowser: TWebBrowser; HTMLCode: string) ;
   var
      sl: TStringList;
      ms: TMemoryStream;
   begin
      WebBrowser.Navigate('about:blank');
      // pretend we're at localhost, so google doesn't complain about the API key
      (WebBrowser.Document as IHTMLDocument2).URL := 'http://localhost/';

      Sleep(50);
      while WebBrowser.ReadyState < READYSTATE_INTERACTIVE do
         Application.ProcessMessages;

      if Assigned(WebBrowser.Document) then
      begin
         sl := TStringList.Create;
         try
            ms := TMemoryStream.Create;
            try
               sl.Text := HTMLCode;
               sl.SaveToStream(ms);
               ms.Seek(0, 0);
               (WebBrowser.Document as IPersistStreamInit).Load(TStreamAdapter.Create(ms));
            finally
               ms.Free;
            end;
         finally
            sl.Free;
         end;
      end;
   end;

const
   rootDoc =
   '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'#13#10 +
   '<html xmlns="http://www.w3.org/1999/xhtml">'#13#10 +
   '<head>'#13#10 +
   '   <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />'#13#10 +
   '   <title>JQuery from Delphi test</title>'#13#10 +
   '</head>'#13#10 +
   '<body>'#13#10 +
   '   <p><a href="#" onclick="$(''#ad'').toggle()">$(''ad'').toggle();</a></p>'#13#10 +
   '   <p><a href="#" onclick="$(''#ad'').animate({opacity:''0.4''}, 750).animate({opacity:''1''}, 1500);">' +
      '$(''#ad'').animate({opacity:''0.4''}, 750).animate({opacity:''1''}, 1500);</a></p>'#13#10 +
   ''#13#10 +
   '   <p><div id="ad" style="background-color: #FFAA00">Some stuff</div></p>'#13#10 +
   '</body>'#13#10 +
   '</html>';

procedure TForm1.btnInjectJQueryClick(Sender: TObject);
begin
   ExecJS(RetrieveJQueryFromResource, false);
end;

procedure TForm1.btnLoadClick(Sender: TObject);
begin
   Forms.Application.ProcessMessages;

   if not Yet then
      WBLoadHTML(WebBrowser1, rootDoc);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
   ExecJS('$("#ad").toggle()');
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
   ExecJS('$(''<div id="stuff">stuff</div>'').appendTo("body");');
end;

procedure TForm1.cbJSKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
var
   s: String;
begin
   if Key = VK_RETURN then
   begin
      s := TCombobox(Sender).Text;
      TCombobox(Sender).Text := '';

      ExecJS(s);
   end;
end;

procedure TForm1.ExecJS(const aCode: String; const output: Boolean = true);
var
   Doc2: IHTMLDocument2;
   Win2: IHTMLWindow2;
begin
   Doc2 := WebBrowser1.Document as IHTMLDocument2;
   Win2 := Doc2.parentWindow;

   if output then
   begin
      // keep a history queue in the Items
      if cbJS.Items.Count > 10 then
         cbJS.Items.Delete(10);
      cbJS.Items.Insert(0, aCode);

      Memo1.Lines.Add(aCode);
   end;

   try
      Win2.execScript(aCode, 'JavaScript');
   except
      Memo1.Lines.Add('JavaScript caused an exception.');
   end;
end;

function TForm1.RetrieveJQueryFromResource: String;
var
   rs: TResourceStream;
   ss: TStringStream;
begin
   rs := TResourceStream.Create(HInstance, 'JQUERY', RT_RCDATA);
   try
      ss := TStringStream.Create;
      try
         ss.LoadFromStream(rs);

         ss.Position := 0;
         Result := ss.DataString;
      finally
         ss.Free;
      end;
   finally
      rs.Free;
   end;
end;

procedure TForm1.SetDropDownWidthToItems(Sender: TObject);
var
   cb: TComboBox;
   i, dropDownWidth, textWidth: Integer;
begin
   cb := TComboBox(Sender);

   dropDownWidth := cb.Width;
   for i := 0 to cb.Items.Count - 1 do
   begin
      textWidth := cb.Canvas.TextWidth(cb.Items[i]);
      if textWidth > dropDownWidth then
         dropDownWidth := textWidth;
   end;

   SendMessage(cb.Handle, CB_SETDROPPEDWIDTH, dropDownWidth, 0);
end;

initialization
   OleInitialize(nil);

finalization
   OleUninitialize;

end.
